#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include "scheduler.h"
#include "sequencer.h"
#include "psg.h"


const uint32_t CYCLE_TIME = 5; // ms

typedef enum {
  ACTION_RESULT_STAY,
  ACTION_RESULT_NEXT,
  ACTION_RESULT_REPEAT,
  ACTION_RESULT_END
} actionResult_t;



sequencerAction_t w(uint32_t wait) {
  sequencerAction_t s = { 
    .type = SA_WAIT, 
    .u.wait = wait 
  };
  return s;
}

sequencerAction_t cs(uint16_t frequency_a, uint8_t volume_a, uint16_t frequency_b, uint8_t volume_b) {
  sequencerAction_t s = { 
    .type = SA_CHANGE_SOUND,
    .u.cs.frequency_a = frequency_a,
    .u.cs.frequency_b = frequency_b,
    .u.cs.volume_a = volume_a,
    .u.cs.volume_b = volume_b
  };
  return s;
}

sequencerAction_t ms(uint16_t frequency_a_0, uint8_t volume_a_0, uint16_t frequency_b_0, uint8_t volume_b_0,
                     uint16_t frequency_a_1, uint8_t volume_a_1, uint16_t frequency_b_1, uint8_t volume_b_1,
                     uint32_t move_time) {
  sequencerAction_t s = { 
    .type = SA_MOVE_SOUND,
    .u.ms.frequency_a_0 = frequency_a_0,
    .u.ms.frequency_b_0 = frequency_b_0,
    .u.ms.volume_a_0 = volume_a_0,
    .u.ms.volume_b_0 = volume_b_0,
    .u.ms.frequency_a_1 = frequency_a_1,
    .u.ms.frequency_b_1 = frequency_b_1,
    .u.ms.volume_a_1 = volume_a_1,
    .u.ms.volume_b_1 = volume_b_1,
    .u.ms.move_time = move_time
  };
  return s;
}

sequencerAction_t cv(uint8_t volume_a, uint8_t volume_b) {
  sequencerAction_t s = {
    .type = SA_CHANGE_VOLUME,
    .u.cv.volume_a = volume_a,
    .u.cv.volume_b = volume_b
  };
  return s;
}

sequencerAction_t mv(uint8_t volume_a_0, uint8_t volume_b_0,
                     uint8_t volume_a_1, uint8_t volume_b_1,
                     uint32_t move_time) {
  sequencerAction_t s = {
    .type = SA_MOVE_VOLUME,
    .u.mv.volume_a_0 = volume_a_0,
    .u.mv.volume_b_0 = volume_b_0,
    .u.mv.volume_a_1 = volume_a_1,
    .u.mv.volume_b_1 = volume_b_1,
    .u.mv.move_time = move_time
  };
  return s;
}

sequencerAction_t r() {
  sequencerAction_t s = {
    .type = SA_REPEAT
  };
  return s;
}

sequencerAction_t e() {
  sequencerAction_t s = {
    .type = SA_END
  };
  return s;
}

static actionResult_t _repeat() {
  return ACTION_RESULT_REPEAT;
}

static actionResult_t _end() {
  return ACTION_RESULT_END;
}

static actionResult_t _wait(uint32_t wait) {
  static bool running = false;
  static uint32_t counter = 0;

  actionResult_t res;
  if (! running) {
    counter = wait;
    res = ACTION_RESULT_STAY;
  } else {
    if (counter == 0) {
      running = false;
      res = ACTION_RESULT_NEXT;
    } else {
      counter -= 1;
      res = ACTION_RESULT_STAY;
    }
  }
  return res;
}

static actionResult_t _change_sound(uint16_t frequency_a, uint8_t volume_a, uint16_t frequency_b, uint8_t volume_b) {
  // set frequency_a and volume_a to generator 0
  // set frequency_b and volume_b to generator 1
  return ACTION_RESULT_NEXT;
}

static bool _change_volume(uint8_t volume_a, uint8_t volume_b) {
  // set volume_a to generator 0
  // set volume_b to generator 1
  return ACTION_RESULT_NEXT;
}

void sequencerStart(sequencerAction_t *sequence) {
}

void sequencerExec(void *handle) {
}

void sequencerInit() {
  schAdd(sequencerExec, NULL, 0, CYCLE_TIME);
}

