#ifndef _SEQUENCER_H_
#define _SEQUENCER_H_


typedef enum {
  SA_IDLE,
  SA_WAIT,
  SA_CHANGE_SOUND,
  SA_MOVE_SOUND,
  SA_CHANGE_VOLUME,
  SA_MOVE_VOLUME,
  SA_REPEAT,
  SA_END,
} sequencerActionType_t;

typedef struct {
  sequencerActionType_t type;
  union {
    uint32_t wait;
    struct {
      uint16_t frequency_a;
      uint8_t volume_a;
      uint16_t frequency_b;
      uint8_t volume_b;
    } cs;
    struct {
      uint16_t frequency_a_0;
      uint8_t volume_a_0;
      uint16_t frequency_b_0;
      uint8_t volume_b_0;
      uint16_t frequency_a_1;
      uint8_t volume_a_1;
      uint16_t frequency_b_1;
      uint8_t volume_b_1;
      uint32_t move_time;
    } ms;
    struct {
      uint8_t volume_a;
      uint8_t volume_b;
    } cv;
    struct {
      uint8_t volume_a_0;
      uint8_t volume_b_0;
      uint8_t volume_a_1;
      uint8_t volume_b_1;
      uint32_t move_time;
    } mv;
  } u;
} sequencerAction_t;

sequencerAction_t w(uint32_t wait);
sequencerAction_t cs(uint16_t frequency_a, uint8_t volume_a, uint16_t frequency_b, uint8_t volume_b);
sequencerAction_t ms(uint16_t frequency_a_0, uint8_t volume_a_0, uint16_t frequency_b_0, uint8_t volume_b_0,
                     uint16_t frequency_a_1, uint8_t volume_a_1, uint16_t frequency_b_1, uint8_t volume_b_1,
                     uint32_t move_time);
sequencerAction_t cv(uint8_t volume_a, uint8_t volume_b);
sequencerAction_t mv(uint8_t volume_a_0, uint8_t volume_b_0,
                     uint8_t volume_a_1, uint8_t volume_b_1,
                     uint32_t move_time);
sequencerAction_t r();
sequencerAction_t e();

void sequencerStart(sequencerAction_t *sequence);

void sequencerInit();


#endif // _SEQUENCER_H_
